@@ -5,40 +5,31 @@
 #
 # Copyright (c) 2007-2008 Peng Huang <shawn.p.huang@gmail.com>
 # Copyright (c) 2009 Hideaki ABE <abe.sendai@gmail.com>
-# Copyright (c) 2010-2012 Takao Fujiwara <takao.fujiwara1@gmail.com>
-# Copyright (c) 2007-2012 Red Hat, Inc.
+# Copyright (c) 2010-2013 Takao Fujiwara <takao.fujiwara1@gmail.com>
+# Copyright (c) 2007-2013 Red Hat, Inc.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2, or (at your option)
-# any later version.
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 __all__ = (
         'ThumbShiftKeyboard',
         'ThumbShiftSegment',
     )
 
+from gi.repository import GLib
 from gi.repository import IBus
 
-try:
-    from gi.repository import Gdk
-    get_default_root_window = Gdk.get_default_root_window
-    property_get = Gdk.property_get
-    intern = Gdk.Atom.intern
-except ImportError:
-    get_default_root_window = lambda : None
-    property_get = lambda : None
-    intern = lambda : None
-
 import segment
 
 _THUMB_BASIC_METHOD = 'base'
@@ -388,6 +379,27 @@ class ThumbShiftKeyboard:
         return (keyval, state)
 
     def __get_xkb_layout(self):
+        # Until Gdk.property_get is fixed
+        '''
+        # Move importing Gdk into ThumbShiftKeyboard from the header
+        # because ibus-engine-anthy --xml does not requre to open X.
+        try:
+            from gi.repository import Gdk
+            get_default_root_window = Gdk.get_default_root_window
+            property_get = Gdk.property_get
+            intern = Gdk.Atom.intern
+        except ImportError:
+            get_default_root_window = lambda : None
+            property_get = lambda : None
+            intern = lambda : None
+        except RuntimeError:
+            # Do we support the engine without display?
+            print >> sys.stderr, "Gdk couldn't be initialized"
+            print >> sys.stderr, 'Could not open display'
+            get_default_root_window = lambda : None
+            property_get = lambda : None
+            intern = lambda : None
+
         root_window = get_default_root_window()
         if not root_window:
             return 0
@@ -416,6 +428,45 @@ class ThumbShiftKeyboard:
             elif data.find('japan:') >= 0:
                 layout = 0
         return layout
+        '''
+
+        layout = 0
+        argv = ['setxkbmap', '-query']
+        (ret, std_out, std_error, exit_status) = \
+                GLib.spawn_sync(None, argv, None,
+                                GLib.SpawnFlags.SEARCH_PATH_FROM_ENVP,
+                                None, None)
+        if not ret:
+            print >> sys.stderr, std_error
+            return layout
+        for line in std_out.split('\n'):
+            if line.startswith('layout:'):
+                data = line.split()[1]
+                if data == 'jp':
+                    layout = 0
+                elif data == 'us':
+                    layout = 1
+            elif line.startswith('options:'):
+                data = line.split()[1]
+                if data.find('japan:nicola_f_bs') >= 0:
+                    layout = 2
+                elif data.find('japan:') >= 0:
+                    layout = 0
+        return 0
+
+    def __reset_layout_and_handakuten(self):
+        mode = self.__prefs.get_value('thumb', 'keyboard_layout_mode')
+        layout = 0
+        if mode == 1:
+            layout = self.__get_xkb_layout()
+        else:
+            layout = self.__prefs.get_value('thumb', 'keyboard_layout')
+        self.set_layout(layout)
+
+        fmv_extension = self.__prefs.get_value('thumb', 'fmv_extension')
+        self.set_fmv_extension(fmv_extension)
+        handakuten = self.__prefs.get_value('thumb', 'handakuten')
+        self.set_handakuten(handakuten)
 
     def reset(self):
         s = self.__prefs.get_value('thumb', 'ls')
@@ -435,18 +486,8 @@ class ThumbShiftKeyboard:
         self.set_t1(t1)
         self.set_t2(t2)
 
-        mode = self.__prefs.get_value('thumb', 'keyboard_layout_mode')
-        layout = 0
-        if mode == 1:
-            layout = self.__get_xkb_layout()
-        else:
-            layout = self.__prefs.get_value('thumb', 'keyboard_layout')
-        self.set_layout(layout)
-
-        fmv_extension = self.__prefs.get_value('thumb', 'fmv_extension')
-        self.set_fmv_extension(fmv_extension)
-        handakuten = self.__prefs.get_value('thumb', 'handakuten')
-        self.set_handakuten(handakuten)
+        GLib.idle_add(self.__reset_layout_and_handakuten,
+                      priority = GLib.PRIORITY_LOW)
 
     def get_ls(self):
         return self.__ls
@@ -534,7 +575,7 @@ class ThumbShiftSegment(segment.Segment):
         super(ThumbShiftSegment, self).__init__(enchars, jachars)
 
     @classmethod
-    def _init_thumb_typing_rule(cls, prefs):
+    def INIT_THUMB_TYPING_RULE(cls, prefs):
         cls._prefs = prefs
         if prefs == None:
             cls._thumb_typing_rule_section = None
