@@ -5,22 +5,22 @@
 #
 # Copyright (c) 2007-2008 Peng Huang <shawn.p.huang@gmail.com>
 # Copyright (c) 2009 Hideaki ABE <abe.sendai@gmail.com>
-# Copyright (c) 2010-2012 Takao Fujiwara <takao.fujiwara1@gmail.com>
-# Copyright (c) 2007-2012 Red Hat, Inc.
+# Copyright (c) 2010-2013 Takao Fujiwara <takao.fujiwara1@gmail.com>
+# Copyright (c) 2007-2013 Red Hat, Inc.
 #
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2, or (at your option)
-# any later version.
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+# You should have received a copy of the GNU General Public License along
+# with this program; if not, write to the Free Software Foundation, Inc.,
+# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
 import sys
 
@@ -37,7 +37,8 @@ class AnthyPrefs(Prefs):
 
     def __init__(self, bus=None, config=None):
         super(AnthyPrefs, self).__init__(bus, config)
-        self.default = _config
+        if _config != None:
+            self.default = _config
         self.set_no_key_warning(True)
         self.fetch_all()
         self.set_no_key_warning(False)
@@ -74,7 +75,7 @@ class AnthyPrefs(Prefs):
         return _japanese_ordered_list
 
     def get_version(self):
-        return '1.5.0'
+        return '1.5.4'
 
     # Convert typing sequences to gsettings key.
     # E.g. 'largea-bracketleft' to 'A['
@@ -102,6 +103,13 @@ class AnthyPrefs(Prefs):
             for _key, _ch in _config_key_to_char.items():
                 self._char_to_config_key[_ch] = _key
         for ch in typing:
+            try:
+                # U+A5 needs to be UTF-8 since gconf values are
+                # disk saved values.
+                ch = ch.encode('utf-8')
+            except:
+                print >> sys.stderr, \
+                    'Failed to encode UTF-8:', ch
             if ch in _supported_gsettings_key_chars:
                 if retval != '':
                     retval += '-'
@@ -270,14 +278,16 @@ _cmd_keys = [
     'on_off',
     'circle_input_mode',
     'circle_kana_mode',
+    'circle_typing_method',
+    'circle_dict_method',
     'latin_mode',
     'wide_latin_mode',
     'hiragana_mode',
     'katakana_mode',
     'half_katakana_mode',
 #    'cancel_pseudo_ascii_mode_key',
-    'circle_typing_method',
-    'circle_dict_method',
+
+    'hiragana_for_latin_with_shift',
 
     'insert_space',
     'insert_alternate_space',
@@ -363,6 +373,7 @@ _config = {
         'half_width_symbol': False,
         'half_width_number': False,
         'half_width_space': False,
+        'latin_with_shift': True,
 
         'shortcut_type': 'default',
 
@@ -1059,13 +1070,15 @@ _config = {
     },
 
     'dict': {
-        'anthy_zipcode': ['/usr/local/share/anthy/zipcode.t'],
-        'ibus_symbol': ['/usr/local/share/ibus-anthy/dicts/symbol.t'],
-        'ibus_oldchar': ['/usr/local/share/ibus-anthy/dicts/oldchar.t'],
+        'zipcode': ['/usr/local/share/ibus-anthy/dicts/zipcode.t'],
+        'symbol': ['/usr/local/share/ibus-anthy/dicts/symbol.t'],
+        'oldchar': ['/usr/local/share/ibus-anthy/dicts/oldchar.t'],
+        'era': ['/usr/local/share/ibus-anthy/dicts/era.t'],
         'files': [
-            '/usr/local/share/anthy/zipcode.t',
+            '/usr/local/share/ibus-anthy/dicts/zipcode.t',
             '/usr/local/share/ibus-anthy/dicts/symbol.t',
             '/usr/local/share/ibus-anthy/dicts/oldchar.t',
+            '/usr/local/share/ibus-anthy/dicts/era.t',
         ],
     },
 
@@ -1092,19 +1105,18 @@ _config = {
         'is_system': True,
     },
 
-    'dict/file/anthy_zipcode': {
+    'dict/file/zipcode': {
         'embed': False,
         'single': True,
         'icon': None,
         'short_label': '〒',
         'long_label': N_("Zip Code Conversion"),
         'preview_lines': 30,
-        'reverse': True,
+        'reverse': False,
         'is_system': True,
-        'encoding': 'euc_jp',
     },
 
-    'dict/file/ibus_symbol': {
+    'dict/file/symbol': {
         'embed': True,
         'single': False,
         'icon': None,
@@ -1115,7 +1127,7 @@ _config = {
         'is_system': True,
     },
 
-    'dict/file/ibus_oldchar': {
+    'dict/file/oldchar': {
         'embed': False,
         'single': True,
         'icon': None,
@@ -1125,12 +1137,23 @@ _config = {
         'reverse': False,
         'is_system': True,
     },
+    'dict/file/era': {
+        'embed': False,
+        'single': True,
+        'icon': None,
+        'short_label': '年',
+        'long_label': N_("Era"),
+        'preview_lines': -1,
+        'reverse': False,
+        'is_system': True,
+    },
 }
 
 _shortcut_default = {
-    'on_off': ['Ctrl+J'],
+    'on_off': ['Zenkaku_Hankaku', 'Ctrl+J'],
     'circle_input_mode': ['Ctrl+comma', 'Ctrl+less'],
-    'circle_kana_mode': ['Ctrl+period', 'Ctrl+greater', 'Hiragana_Katakana'],
+    # Removed Hiragana_Katakana key in circle_kana_mode for latin_with_shift.
+    'circle_kana_mode': ['Ctrl+period', 'Ctrl+greater'],
 #    'cancel_pseudo_ascii_mode_key': ['Escape'],
     'circle_typing_method': ['Alt+Romaji', 'Ctrl+slash'],
     'circle_dict_method': ['Alt+Henkan'],
@@ -1187,6 +1210,8 @@ _shortcut_default = {
 
     'dict_admin': ['F11'],
     'add_word': ['F12'],
+
+    'hiragana_for_latin_with_shift': ['Hiragana_Katakana'],
 }
 
 _config['shortcut/default'] = dict.fromkeys(_cmd_keys, [])
@@ -1246,7 +1271,10 @@ _shortcut_atok = {
     'convert_to_wide_latin': ['F9', 'Ctrl+P'],
     'convert_to_latin': ['F10', 'Ctrl+at'],
 
+    'dict_admin': ['F11'],
     'add_word': ['Ctrl+F7'],
+
+    'hiragana_for_latin_with_shift': ['Ctrl+J'],
 }
 
 _config['shortcut/atok'] = dict.fromkeys(_cmd_keys, [])
@@ -1296,6 +1324,11 @@ _shortcut_wnn = {
     'convert_to_half': ['F8'],
     'convert_to_wide_latin': ['F9'],
     'convert_to_latin': ['F10'],
+
+    'dict_admin': ['F11'],
+    'add_word': ['F12'],
+
+    'hiragana_for_latin_with_shift': ['Hiragana_Katakana'],
 }
 
 _config['shortcut/wnn'] = dict.fromkeys(_cmd_keys, [])
